package ca.canadapost.cpcdp.returns;

import java.util.Map;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.interceptor.LoggingInInterceptor;
import org.apache.cxf.interceptor.LoggingOutInterceptor;

import ca.canadapost.cpcdp.returns.stub.openreturn.GetOpenReturnTemplatesRequest;
import ca.canadapost.cpcdp.returns.stub.openreturn.GetOpenReturnTemplatesResponse;
import ca.canadapost.cpcdp.returns.stub.openreturn.Messages;
import ca.canadapost.cpcdp.returns.stub.openreturn.OpenReturnPortType;
import ca.canadapost.cpcdp.returns.stub.openreturn.OpenReturnService;
import ca.canadapost.cpcdp.returns.util.ClientUtil;

/**
 * Sample code for the GetOpenReturnTemplates Canada Post service.
 * <p>
 * The GetOpenReturnTemplates service is used to retrieve the list of active open
 * return templates (based on the mailed-on-behalf-of customer number). Active
 * open return templates are those that still have open return artifacts available
 * for retrieval. Once all of the artifacts from a template have been retrieved,
 * the open return template is consider depleted, and is deleted.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetOpenReturnTemplatesClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
            // Your username, password and customer number are imported from the following file: CPCWS_SOAP_Returns_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            OpenReturnService service = new OpenReturnService();
            OpenReturnPortType port = service.getOpenReturnPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/openreturn/v2");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //log request/response
            //client.getInInterceptors().add(new LoggingInInterceptor());
            //client.getOutInterceptors().add(new LoggingOutInterceptor());
            
            //create request
            GetOpenReturnTemplatesRequest request = new GetOpenReturnTemplatesRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setFrom(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-05-20"));
            //request.setTo(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-05-25"));

            //execute request
            GetOpenReturnTemplatesResponse response = port.getOpenReturnTemplates(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getTemplates() != null && response.getTemplates().getTemplateId().size() > 0){
                    for(String id : response.getTemplates().getTemplateId()){
                        System.out.println("Template Id: " + id);
                    }
                } else {
                    System.out.println("No templates returned.");
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
